/* Copyright (c) 2011 Boulder Real Time Technologies, Inc. */
/* All rights reserved */

#ifndef __buplot__
#define __buplot__

/* This software module is wholly owned by Boulder Real Time 
	Technologies, Inc. Any use of this software module without
	express written permission from Boulder Real Time Technologies,
	Inc. is prohibited. */

#ifdef  __cplusplus
extern "C" {
#endif

#include "tk.h"
#include "X11/Xutil.h"

#include "brttutil.h"
#include "stock.h"
#include "coords.h"

#define BUPLOT_SOUTHWEST	0
#define BUPLOT_SOUTHEAST	1
#define BUPLOT_NORTHWEST	2
#define BUPLOT_NORTHEAST	3

typedef struct BpviewportItem {
	Tk_Item header;
	double x, y;
	int w, h;
	int spc_w, spc_h;
	int mleft, mright, mbottom, mtop; 	/* Margins */
	int xoffset, yoffset;
	int xgain_anchor, ygain_anchor;
	double xgain, ygain;
	Tk_Anchor anchor;
	Tcl_Interp *interp;
	Tk_Canvas canvas;
	GC gc;
	XColor *background;
	XColor *background_frame;
	double x0, y0;
	double latr, lonr;
	double bblatmin, bblatmax, bblonmin, bblonmax;
	double xleft, xright, ybottom, ytop;
	double spc_xleft, spc_xright, spc_ybottom, spc_ytop;
	int xmode, ymode, wtran;
	double xmin, xmax, ymin, ymax;
	char *name;
	Tbl *slaves;
} BpviewportItem;

extern void buplot_buffer_draws (Display *display, Drawable drawable, GC gc, Tcl_Interp *interp, 
			Tk_Canvas canvas, int postscript, int fill, int flush, 
			BpviewportItem *vp, int w, int h, double fx1, double fy1, double fx2, double fy2);

extern void buplot_init (int postscript, Tcl_Interp *interp, int dict);

extern int buplot_uninit (int postscript, Tcl_Interp *interp);

extern int buplot_setup_line (Display *display, GC gc, Tcl_Interp *interp, Tk_Canvas canvas, 
			int postscript, XColor *color, int linewidth, int linestyle, int joinstyle);

extern int buplot_setup_fill (Display * display, GC gc, Tcl_Interp * interp, Tk_Canvas canvas, int postscript,
		  XColor * color);

extern int buplot_poly_findclosest (BpviewportItem * viewport, int npts, double *xpts, double *ypts, 
							int x, int y, int *imin, double *dmin);

extern int buplot_poly_draw (Display * display, Drawable drawable, GC gc, Tcl_Interp * interp, Tk_Canvas canvas, int postscript,
	BpviewportItem * viewport, int npts, double *xpts, double *ypts, int fill, int complete, int ignoretran);

extern int buplot_get_nice_fvals(double start,double end,double width,double minbigwidth,
		double *start_big,int *n_big,double *incr_big,
		double *start_sml,int *n_sml,double *incr_sml);

extern int buplot_get_scales (BpviewportItem *vp, int *w, int *h, double *xtsart, double *xend, double *xscale, double *ystart, double *yend, double *yscale);

extern int buplot_get_xpixel (BpviewportItem * vp, char *xstr, double *xpixels);

extern int buplot_get_ypixel (BpviewportItem * vp, char *ystr, double *ypixels);

extern int buplot_get_pixels (BpviewportItem *vp, char *xstr, char *ystr, double *xpixels, double *ypixels);

extern int buplot_get_wcoords (BpviewportItem *vp, double x, double y, double *xcoord, double *ycoord);

extern int buplot_wcoords2pixels (BpviewportItem * vp, double xcoord, double ycoord, double *x, double *y);

extern int buplot_translate_and_gain (BpviewportItem * vp, double dx_translate, double dy_translate,
			int xgain_anchor, int ygain_anchor, double xgain, double ygain);

extern int buplot_setup_text (Display * display, GC gc, Tcl_Interp * interp, Tk_Canvas canvas, int postscript,
		  XColor * color, Tk_Font font);

extern int buplot_get_metrics (Tk_Font font, char *text, int *w, int *h);

extern int buplot_parse_justification (char *just, double *jx, double *jy);

extern void buplot_draw_text (Display *display, Drawable drawable, GC gc, Tcl_Interp *interp, 
		Tk_Canvas canvas, int postscript, BpviewportItem *vp, Tk_Font font, 
		double x, double y, char *just, double angle, char *text);

extern void buplot_scaletext_draw (Display *display, Drawable drawable, GC gc, 
			Tcl_Interp *interp, Tk_Canvas canvas, int postscript,
			BpviewportItem *viewport, double xpt, double ypt, 
			double xsh, double ysh, Tk_Font font, char *justification, double angle, char *text);

extern int buplot_nice_number (double number, char *format, char *string);

extern void buplot_draw_xaxis_stuff (Display *display, Drawable drawable, GC gc, BpviewportItem *vp, int postscript,
	XColor *color, int linewidth, int linestyle, double incr, double fy1, double fy2,
	XColor *color_small, int linewidth_small, int linestyle_small, double incr_small, 
	double fy1_small, double fy2_small, double mindx);

extern void buplot_draw_yaxis_stuff (Display *display, Drawable drawable, GC gc, BpviewportItem *vp, int postscript,
	XColor *color, int linewidth, int linestyle, double incr, double fy1, double fy2,
	XColor *color_small, int linewidth_small, int linestyle_small, double incr_small, 
	double fy1_small, double fy2_small, double mindy);

extern void buplot_latlon2edp (double latr, double lonr, double lat, double lon, double *delx, double *dely);
extern void buplot_latlon2edp2 (double latr, double lonr, double lat, double lon, double *delx, double *dely, double *north);

extern void buplot_edp2latlon (double latr, double lonr, double delx, double dely, double *lat, double *lon);

extern int buplot_colorname_to_xcolor (Tcl_Interp *interp, Tk_Window tkwin, Pf *pf, char *colorname, XColor **colorcell);
extern int buplot_colorname_to_pixel (Tcl_Interp *interp, Tk_Window tkwin, Pf *pf, char *colorname, unsigned long *pixel);
extern int buplot_colorname_to_pixelbg (Tcl_Interp *interp, Tk_Window tkwin, Pf *pf, char *colorname, 
							unsigned long *pixel, char *xcolorname);
extern int buplot_transform_colorname (char *colorname, char *xcolorname);
extern int buplot_fontname_to_font (Tcl_Interp *interp, Tk_Window tkwin, Pf *pf, char *fontname, 
							char *fontname_backup, Tk_Font *font);
extern int buplot_fontmetrics (Tk_Font tkfont, char *string, 
			int *str_w, int *str_h, int *ascent, int *descent);
extern int buplot_hls_to_rgb(double h, double l, double s, double gamma, double *r, double *g, double *b);

extern int buplot_rgb_to_hls(double r, double g, double b, double *h, double *l, double *s);

extern int buplot_draw_colorcontour (Display *display, Drawable drawable, GC gc, Tcl_Interp *interp, Tk_Canvas canvas, 
			int postscript, BpviewportItem *vp, XImage **default_ximage, char *blend_mode, 
			double zmin, double zmax, double dz, XColor *colormin, XColor *colormax,
			char *interp_mode, int gtran, double slat, double slon, double sdepth, 
			char *method, char *model, char *phase, double rlat, double rlon, double xstrike, 
			BUGrid *grid, int isamp, XImage **backgroundimage, float **xcoords, float **ycoords);

extern XImage *buplot_image_fill (Display *display, Visual *visual, int depth, 
						unsigned char *pixelcolors, int w, int h);

extern int buplot_make_xtime_axes (BpviewportItem * vp, Tk_Font font, Display * display, GC gc, 
			Drawable drawable, int postscript,
			XColor *linecolor, int linewidth, int linestyle,
			XColor *fontcolor, char *label_time_format, char *label_data_format);

extern int buplot_qplot (double ts, double dt, int nsamp, void *vdata, int isdouble, double tmin, double twin, 
		int npixels, int x, int y, int h, double *ybot, double *ytop, 
		int scale_type, double ygain, XPoint **xp, double **xxpl, double **yypl, float **color);
		
extern void buplot_tcl_clear_register(Tcl_Interp *interp, int printflag);

extern BpviewportItem *buplot_viewport_get_viewport (char *name);
extern int buplot_viewport_set_viewport (char *name, BpviewportItem *viewport);
extern BpviewportItem *buplot_viewport_register_slave (char *viewport, char *type, void *ptr, void (*delete_proc) (), 
		int (*findclosest_proc) (), double xmin, double xmax, double ymin, double ymax);
extern int buplot_viewport_delete_slave (char *viewport, void *ptr);
extern int buplot_viewport_locate (Window window, int x, int y, BpviewportItem **vp, int *inside, Tbl **items);
extern void buplot_viewport_free (BpviewportItem *viewport);
extern double buplot_viewport_point (BpviewportItem *viewport, double *pointPtr);
extern int buplot_viewport_set_xscale (BpviewportItem *viewport, double xleft, double xright);

extern void buplot_latlon2edp (double latr, double lonr, double lat, double lon, double *delx, double *dely);
extern void buplot_edp2latlon (double latr, double lonr, double delx, double dely, double *lat, double *lon);
extern int buplot_mapopen (void);
extern int buplot_mapselect (char *types, char *resolution, double minlon, double maxlon, double minlat, double maxlat,
								double lonref, double latref);
extern int buplot_mapnextseg (int *npts, int *npts_size, float **lon, float **lat, char *type, int *level, int *antipode);
extern void buplot_sleep (double sleep);
extern void buplot_queue_event (Tcl_ThreadId targetId, int (*callback) (void *user_data), void *user_data, 
									double timer, int *pending, unsigned long *token);
extern void buplot_delete_event (unsigned long token);
extern int buplot_run_python (char *proc, char *args, char **ret);
extern int buplot_run_python_tcl (Tcl_ThreadId tcl_thread, int wait, char *proc, char *args, double delay, int *pending, char **ret);
extern int buplot_was_set (char *option, Tk_ConfigSpec *specs, int argc, Tcl_Obj *CONST objv[]);
extern int buplot_pythonstr2pf (char *str, char *name, Pf **pf);
extern int buplot_pfstr2pf (char *str, char *name, Pf **pf);
extern void buplot_canvas_coords (BpviewportItem *vp, int justify, int x, int y, int *xc, int *yc);
extern int buplot_parse_string (ClientData clientData, Tcl_Interp *interp,
		Tk_Window tkwin, char const *value, char *widgRec, int offset);
extern const char *buplot_print_string (ClientData clientData, 
                Tk_Window tkwin, char *widgRec, int offset, Tcl_FreeProc **freeProcPtr);


#ifdef  __cplusplus
}
#endif

#endif
